@@ -2,16 +2,20 @@
 JC cli module
 """
 
+import io
 import sys
 import os
 import textwrap
 import signal
 import shlex
 import subprocess
-import json
+from typing import List, Dict
 from .lib import (__version__, parser_info, all_parser_info, parsers,
-                  _get_parser, _parser_is_streaming)
+                  _get_parser, _parser_is_streaming, standard_parser_mod_list,
+                  plugin_parser_mod_list, streaming_parser_mod_list)
 from . import utils
+from .cli_data import long_options_map
+from .shell_completions import bash_completion, zsh_completion
 from . import tracebackplus
 from .exceptions import LibraryNotInstalled, ParseError
 
@@ -21,13 +25,12 @@ try:
     from pygments import highlight
     from pygments.style import Style
     from pygments.token import (Name, Number, String, Keyword)
-    from pygments.lexers import JsonLexer
+    from pygments.lexers.data import JsonLexer, YamlLexer
     from pygments.formatters import Terminal256Formatter
     PYGMENTS_INSTALLED = True
 except Exception:
     PYGMENTS_INSTALLED = False
 
-
 JC_ERROR_EXIT = 100
 
 
@@ -134,7 +137,7 @@ def set_env_colors(env_colors=None):
 
 def piped_output(force_color):
     """
-    Return False if stdout is a TTY. True if output is being piped to
+    Return False if `STDOUT` is a TTY. True if output is being piped to
     another program and foce_color is True. This allows forcing of ANSI
     color codes even when using pipes.
     """
@@ -166,6 +169,22 @@ def parsers_text(indent=0, pad=0):
     return ptext
 
 
+def options_text(indent=0, pad=0):
+    """Return the argument and description information from each option"""
+    otext = ''
+    padding_char = ' '
+    for option in long_options_map:
+        o_short = '-' + long_options_map[option][0]
+        o_desc = long_options_map[option][1]
+        o_combined = o_short + ',  ' + option
+        padding = pad - len(o_combined)
+        indent_text = padding_char * indent
+        padding_text = padding_char * padding
+        otext += indent_text + o_combined + padding_text + o_desc + '\n'
+
+    return otext
+
+
 def about_jc():
     """Return jc info and the contents of each parser.info as a dictionary"""
     return {
@@ -180,48 +199,44 @@ def about_jc():
         'python_version': '.'.join((str(sys.version_info.major), str(sys.version_info.minor), str(sys.version_info.micro))),
         'python_path': sys.executable,
         'parser_count': len(all_parser_info()),
+        'standard_parser_count': len(standard_parser_mod_list()),
+        'streaming_parser_count': len(streaming_parser_mod_list()),
+        'plugin_parser_count': len(plugin_parser_mod_list()),
         'parsers': all_parser_info()
     }
 
 
 def helptext():
     """Return the help text with the list of parsers"""
-    parsers_string = parsers_text(indent=12, pad=17)
+    parsers_string = parsers_text(indent=4, pad=20)
+    options_string = options_text(indent=4, pad=20)
 
     helptext_string = f'''\
-    jc converts the output of many commands and file-types to JSON
+jc converts the output of many commands and file-types to JSON or YAML
 
-    Usage:  COMMAND | jc PARSER [OPTIONS]
+Usage:
+    COMMAND | jc PARSER [OPTIONS]
 
-            or magic syntax:
+    or magic syntax:
 
-            jc [OPTIONS] COMMAND
+    jc [OPTIONS] COMMAND
 
-    Parsers:
+Parsers:
 {parsers_string}
-    Options:
-            -a    about jc
-            -C    force color output even when using pipes (overrides -m)
-            -d    debug (-dd for verbose debug)
-            -h    help (-h --parser_name for parser documentation)
-            -m    monochrome output
-            -p    pretty print output
-            -q    quiet - suppress parser warnings (-qq to ignore streaming errors)
-            -r    raw JSON output
-            -u    unbuffer output
-            -v    version info
+Options:
+{options_string}
+Examples:
+    Standard Syntax:
+        $ dig www.google.com | jc --dig --pretty
 
-    Examples:
-            Standard Syntax:
-                $ dig www.google.com | jc --dig -p
+    Magic Syntax:
+        $ jc --pretty dig www.google.com
 
-            Magic Syntax:
-                $ jc -p dig www.google.com
+    Parser Documentation:
+        $ jc --help --dig
+'''
 
-            Parser Documentation:
-                $ jc -h --dig
-    '''
-    return textwrap.dedent(helptext_string)
+    return helptext_string
 
 
 def help_doc(options):
@@ -263,11 +278,52 @@ def versiontext():
     return textwrap.dedent(versiontext_string)
 
 
+def yaml_out(data, pretty=False, env_colors=None, mono=False, piped_out=False, ascii_only=False):
+    """
+    Return a YAML formatted string. String may include color codes. If the
+    YAML library is not installed, output will fall back to JSON with a
+    warning message to STDERR"""
+    # make ruamel.yaml import optional
+    try:
+        from ruamel.yaml import YAML
+        YAML_INSTALLED = True
+    except Exception:
+        YAML_INSTALLED = False
+
+    if YAML_INSTALLED:
+        y_string_buf = io.BytesIO()
+        # monkey patch to disable plugins since we don't use them and in
+        # ruamel.yaml versions prior to 0.17.0 the use of __file__ in the
+        # plugin code is incompatible with the pyoxidizer packager
+        YAML.official_plug_ins = lambda a: []
+        yaml = YAML()
+        yaml.default_flow_style = False
+        yaml.explicit_start = True
+        yaml.allow_unicode = not ascii_only
+        yaml.encoding = 'utf-8'
+        yaml.dump(data, y_string_buf)
+        y_string = y_string_buf.getvalue().decode('utf-8')[:-1]
+
+        if not mono and not piped_out:
+            # set colors
+            class JcStyle(Style):
+                styles = set_env_colors(env_colors)
+
+            return str(highlight(y_string, YamlLexer(), Terminal256Formatter(style=JcStyle))[0:-1])
+
+        return y_string
+
+    utils.warning_message(['YAML Library not installed. Reverting to JSON output.'])
+    return json_out(data, pretty=pretty, env_colors=env_colors, mono=mono, piped_out=piped_out, ascii_only=ascii_only)
+
+
 def json_out(data, pretty=False, env_colors=None, mono=False, piped_out=False, ascii_only=False):
     """
     Return a JSON formatted string. String may include color codes or be
     pretty printed.
     """
+    import json
+
     separators = (',', ':')
     indent = None
 
@@ -287,24 +343,42 @@ def json_out(data, pretty=False, env_colors=None, mono=False, piped_out=False, a
     return j_string
 
 
-def safe_print_json(list_or_dict, pretty=None, env_colors=None, mono=None,
-                    piped_out=None, flush=None):
-    """Safely prints JSON output in both UTF-8 and ASCII systems"""
-    try:
-        print(json_out(list_or_dict,
-                       pretty=pretty,
-                       env_colors=env_colors,
-                       mono=mono,
-                       piped_out=piped_out),
-              flush=flush)
-    except UnicodeEncodeError:
-        print(json_out(list_or_dict,
-                       pretty=pretty,
-                       env_colors=env_colors,
-                       mono=mono,
-                       piped_out=piped_out,
-                       ascii_only=True),
-              flush=flush)
+def safe_print_out(list_or_dict, pretty=None, env_colors=None, mono=None,
+                   piped_out=None, flush=None, yaml=None):
+    """Safely prints JSON or YAML output in both UTF-8 and ASCII systems"""
+    if yaml:
+        try:
+            print(yaml_out(list_or_dict,
+                           pretty=pretty,
+                           env_colors=env_colors,
+                           mono=mono,
+                           piped_out=piped_out),
+                  flush=flush)
+        except UnicodeEncodeError:
+            print(yaml_out(list_or_dict,
+                           pretty=pretty,
+                           env_colors=env_colors,
+                           mono=mono,
+                           piped_out=piped_out,
+                           ascii_only=True),
+                  flush=flush)
+
+    else:
+        try:
+            print(json_out(list_or_dict,
+                           pretty=pretty,
+                           env_colors=env_colors,
+                           mono=mono,
+                           piped_out=piped_out),
+                  flush=flush)
+        except UnicodeEncodeError:
+            print(json_out(list_or_dict,
+                           pretty=pretty,
+                           env_colors=env_colors,
+                           mono=mono,
+                           piped_out=piped_out,
+                           ascii_only=True),
+                  flush=flush)
 
 
 def magic_parser(args):
@@ -318,7 +392,7 @@ def magic_parser(args):
         jc_options    (list)  list of jc options
     """
     # bail immediately if there are no args or a parser is defined
-    if len(args) <= 1 or args[1].startswith('--'):
+    if len(args) <= 1 or (args[1].startswith('--') and args[1] not in long_options_map):
         return False, None, None, []
 
     args_given = args[1:]
@@ -326,6 +400,12 @@ def magic_parser(args):
 
     # find the options
     for arg in list(args_given):
+        # long option found - populate option list
+        if arg in long_options_map:
+            options.extend(long_options_map[arg][0])
+            args_given.pop(0)
+            continue
+
         # parser found - use standard syntax
         if arg.startswith('--'):
             return False, None, None, []
@@ -420,6 +500,9 @@ def main():
     # find options if magic_parser did not find a command
     if not valid_command:
         for opt in sys.argv:
+            if opt in long_options_map:
+                options.extend(long_options_map[opt][0])
+
             if opt.startswith('-') and not opt.startswith('--'):
                 options.extend(opt[1:])
 
@@ -435,6 +518,9 @@ def main():
     raw = 'r' in options
     unbuffer = 'u' in options
     version_info = 'v' in options
+    yaml_out = 'y' in options
+    bash_comp = 'B' in options
+    zsh_comp = 'Z' in options
 
     if verbose_debug:
         tracebackplus.enable(context=11)
@@ -443,11 +529,12 @@ def main():
         mono = True
 
     if about:
-        safe_print_json(about_jc(),
-                        pretty=pretty,
-                        env_colors=jc_colors,
-                        mono=mono,
-                        piped_out=piped_output(force_color))
+        safe_print_out(about_jc(),
+                       pretty=pretty,
+                       env_colors=jc_colors,
+                       mono=mono,
+                       piped_out=piped_output(force_color),
+                       yaml=yaml_out)
         sys.exit(0)
 
     if help_me:
@@ -458,6 +545,14 @@ def main():
         utils._safe_print(versiontext())
         sys.exit(0)
 
+    if bash_comp:
+        utils._safe_print(bash_completion())
+        sys.exit(0)
+
+    if zsh_comp:
+        utils._safe_print(zsh_completion())
+        sys.exit(0)
+
     # if magic syntax used, try to run the command and error if it's not found, etc.
     magic_stdout, magic_stderr, magic_exit_code = None, None, 0
     if run_command:
@@ -534,12 +629,13 @@ def main():
                                   quiet=quiet,
                                   ignore_exceptions=ignore_exceptions)
             for line in result:
-                safe_print_json(line,
-                                pretty=pretty,
-                                env_colors=jc_colors,
-                                mono=mono,
-                                piped_out=piped_output(force_color),
-                                flush=unbuffer)
+                safe_print_out(line,
+                               pretty=pretty,
+                               env_colors=jc_colors,
+                               mono=mono,
+                               piped_out=piped_output(force_color),
+                               flush=unbuffer,
+                               yaml=yaml_out)
 
             sys.exit(combined_exit_code(magic_exit_code, 0))
 
@@ -550,12 +646,13 @@ def main():
                                   raw=raw,
                                   quiet=quiet)
 
-            safe_print_json(result,
-                            pretty=pretty,
-                            env_colors=jc_colors,
-                            mono=mono,
-                            piped_out=piped_output(force_color),
-                            flush=unbuffer)
+            safe_print_out(result,
+                           pretty=pretty,
+                           env_colors=jc_colors,
+                           mono=mono,
+                           piped_out=piped_output(force_color),
+                           flush=unbuffer,
+                           yaml=yaml_out)
 
         sys.exit(combined_exit_code(magic_exit_code, 0))
 
@@ -570,14 +667,6 @@ def main():
         ])
         sys.exit(combined_exit_code(magic_exit_code, JC_ERROR_EXIT))
 
-    except json.JSONDecodeError:
-        if debug:
-            raise
-
-        utils.error_message(['There was an issue generating the JSON output.',
-                             'For details use the -d or -dd option.'])
-        sys.exit(combined_exit_code(magic_exit_code, JC_ERROR_EXIT))
-
     except Exception:
         if debug:
             raise
