@@ -1,7 +1,7 @@
-"""jc - JSON Convert `pidstat` command output streaming parser
+"""jc - JSON Convert `pidstat -h` command output streaming parser
 
 > This streaming parser outputs JSON Lines (cli) or returns an Iterable of
-  Dictionaries (module)
+> Dictionaries (module)
 
 Must use the `-h` option in `pidstat`. All other `pidstat` options are
 supported in combination with `-h`.
@@ -11,11 +11,11 @@ Usage (cli):
     $ pidstat | jc --pidstat-s
 
 > Note: When piping `jc` converted `pidstat` output to other processes it
-  may appear the output is hanging due to the OS pipe buffers. This is
-  because `pidstat` output is too small to quickly fill up the buffer. Use
-  the `-u` option to unbuffer the `jc` output if you would like immediate
-  output. See the [readme](https://github.com/kellyjonbrazil/jc/tree/master#unbuffering-output)
-  for more information.
+> may appear the output is hanging due to the OS pipe buffers. This is
+> because `pidstat` output is too small to quickly fill up the buffer. Use
+> the `-u` option to unbuffer the `jc` output if you would like immediate
+> output. See the [readme](https://github.com/kellyjonbrazil/jc/tree/master#unbuffering-output)
+> for more information.
 
 Usage (module):
 
@@ -84,7 +84,7 @@ from jc.exceptions import ParseError
 class info():
     """Provides parser metadata (version, author, etc.)"""
     version = '1.0'
-    description = '`pidstat` command streaming parser'
+    description = '`pidstat -h` command streaming parser'
     author = 'Kelly Brazil'
     author_email = 'kellyjonbrazil@gmail.com'
     compatible = ['linux']
